#!/usr/bin/env node
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};

// src/index.ts
var import_node_path3 = __toESM(require("path"));
var import_chalk2 = __toESM(require("chalk"));
var import_commander = require("commander");
var import_dotenv = require("dotenv");

// src/dump.ts
var import_node_fs2 = __toESM(require("fs"));
var import_node_path2 = __toESM(require("path"));
var import_leetcode_query = require("leetcode-query");
var import_node_fetch = __toESM(require("node-fetch"));
var import_ora = __toESM(require("ora"));

// src/constants.ts
var EXTS = {
  cpp: ".cpp",
  java: ".java",
  python: ".py",
  python3: ".py",
  c: ".c",
  csharp: ".cs",
  javascript: ".js",
  ruby: ".rb",
  swift: ".swift",
  golang: ".go",
  scala: ".scala",
  kotlin: ".kt",
  rust: ".rs",
  php: ".php",
  typescript: ".ts",
  racket: ".rkt",
  erlang: ".erl",
  elixir: ".ex"
};
var COMMENTS = {
  cpp: "//",
  java: "//",
  python: "#",
  python3: "#",
  c: "//",
  csharp: "//",
  javascript: "//",
  ruby: "#",
  swift: "//",
  golang: "//",
  scala: "//",
  kotlin: "//",
  rust: "//",
  php: "//",
  typescript: "//",
  racket: ";",
  erlang: "%",
  elixir: "#"
};
var LEETCODE_BASE = "https://leetcode.com";
var LEETCODE_DETAIL_BASE = LEETCODE_BASE + "/submissions/detail/";
var LEETCODE_SLUG_BASE = LEETCODE_BASE + "/problems/";
var README_TEMPLATE = `
# LeetCode Solutions

$card

[*LeetCode Stats Card*](https://github.com/JacobLinCool/LeetCode-Stats-Card)

## Solutions

| Problem | Difficulty | Tags | Solution |
| ------- | ---------- | ---- | -------- |
$table
`;

// src/utils.ts
var import_node_fs = __toESM(require("fs"));
var import_node_path = __toESM(require("path"));
var import_chalk = __toESM(require("chalk"));
function get_package_meta() {
  return JSON.parse(import_node_fs.default.readFileSync(import_node_path.default.join(__dirname, "../package.json"), "utf8"));
}
function readable_memory(mem) {
  const units = ["B", "KB", "MB", "GB", "TB"];
  let unit = 0;
  while (mem > 1e3) {
    mem /= 1e3;
    unit++;
  }
  return `${mem.toFixed(2)} ${units[unit]}`;
}
function sleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}
async function retry(func, max_retry2 = 3) {
  let error = null;
  for (let retry2 = 0; retry2 < max_retry2; retry2++) {
    try {
      return await func();
    } catch (err) {
      console.error(import_chalk.default.redBright(err));
      error = err;
    }
  }
  throw error;
}

// src/dump.ts
var leetcode;
var max_retry = 3;
async function dump(session, output, clean, cooldown, timezone, pure, max) {
  const { dir } = await setup({ timezone, max, output, clean, session });
  const spinner = (0, import_ora.default)({ text: "Scanning...", spinner: "bouncingBar" }).start();
  const { list, ac } = await get_list();
  spinner.succeed(`Scan Done. (${list.length} Problems, ${ac.length} Accepted)`);
  await sleep(cooldown);
  const table = [];
  spinner.start("Dumping Submissions...");
  for (let i = 0; i < ac.length; i++) {
    spinner.text = `Dumping Submissions... (${i + 1}/${ac.length})`;
    const { titleSlug: slug } = ac[i];
    const problem = await get_problem(slug);
    const folder = import_node_path2.default.resolve(dir, `${problem.questionFrontendId}. ${problem.title}`);
    if (!import_node_fs2.default.existsSync(folder)) {
      import_node_fs2.default.mkdirSync(folder, { recursive: true });
    }
    import_node_fs2.default.writeFileSync(import_node_path2.default.resolve(folder, "README.md"), `# ${problem.questionFrontendId}. ${problem.title}

Tags: ${problem.topicTags.map((t) => "`" + t.name + "`").join(", ")}

${problem.content}`);
    import_node_fs2.default.writeFileSync(import_node_path2.default.resolve(folder, "NOTE.md"), problem.note);
    const submissions = await get_submissions(slug);
    spinner.text += ` [${Object.keys(submissions).join(", ")}]`;
    await sleep(cooldown);
    const row = [
      `[${problem.questionFrontendId}. ${problem.title}](./${encodeURI(import_node_path2.default.basename(folder))}) [\u{1F517}](${LEETCODE_SLUG_BASE}${slug}/)`,
      problem.difficulty,
      problem.topicTags.map((t) => "`" + t.name + "`").join(", "),
      []
    ];
    for (const lang in submissions) {
      const ext = EXTS[lang];
      const file = import_node_path2.default.resolve(folder, `${slug}${ext}`);
      if (!import_node_fs2.default.existsSync(file)) {
        const submission = await get_submission(submissions[lang].id);
        const info = `${COMMENTS[lang]} ${problem.questionFrontendId}. ${problem.title} (${new Date(+submissions[lang].timestamp * 1e3).toLocaleDateString()})
${COMMENTS[lang]} Runtime: ${submission.runtime} ms (${submission.runtime_percentile.toFixed(2)}%) Memory: ${readable_memory(submission.memory)} (${submission.memory_percentile.toFixed(2)}%) 

`;
        import_node_fs2.default.writeFileSync(file, `${pure ? "" : info}${submission.code}`);
        row[3].push(`[${lang}](${encodeURI(`./${import_node_path2.default.basename(folder)}/${slug}${ext}`)})`);
        await sleep(cooldown);
      }
    }
    row[3].sort();
    table.push(row);
  }
  spinner.succeed("Submissions Dumped.");
  import_node_fs2.default.writeFileSync(import_node_path2.default.resolve(dir, "README.md"), await create_toc(table));
}
async function create_toc(table) {
  const username = await get_username();
  const card = `[![LeetCode Stats Card](https://leetcode.card.workers.dev/?username=${username})](https://leetcode.com/${username}/)`;
  const table_str = table.map(([title, difficulty, tags, solutions]) => `| ${title} | ${difficulty} | ${tags} | ${solutions.join(" \\| ")} |`).join("\n");
  return README_TEMPLATE.replace("$card", card).replace("$table", table_str);
}
async function setup({
  session,
  output,
  clean,
  timezone,
  max
}) {
  process.env.TZ = timezone;
  max_retry = max;
  const dir = import_node_path2.default.resolve(output);
  if (import_node_fs2.default.existsSync(dir) && clean) {
    import_node_fs2.default.rmSync(dir, { recursive: true });
  }
  if (!import_node_fs2.default.existsSync(dir)) {
    import_node_fs2.default.mkdirSync(dir, { recursive: true });
  }
  const credential = new import_leetcode_query.Credential();
  await credential.init(session);
  leetcode = new import_leetcode_query.LeetCode(credential);
  return { dir };
}
async function get_list() {
  const ql = `query allQuestionsStatusesRaw { allQuestions: allQuestionsRaw { titleSlug questionId status } }`;
  const json = await retry(() => leetcode.graphql({
    query: ql,
    operationName: "allQuestionsStatusesRaw",
    variables: {}
  }), max_retry);
  const list = json.data.allQuestions;
  const ac = list.filter((item) => item.status === "ac").sort((a, b) => +a.questionId - +b.questionId);
  return { list, ac };
}
async function get_problem(slug) {
  const ql = `
    query questionData($slug: String!) {
        question(titleSlug: $slug) {
            questionFrontendId title content difficulty similarQuestions categoryTitle stats topicTags { name slug } note
        }
    }`;
  const json = (await retry(() => leetcode.graphql({ query: ql, operationName: "questionData", variables: { slug } }), max_retry)).data.question;
  json.similarQuestions = JSON.parse(json.similarQuestions);
  json.stats = JSON.parse(json.stats);
  return json;
}
async function get_submissions(slug) {
  const ql = `
    query Submissions($offset: Int!, $limit: Int!, $last: String, $slug: String!) {
        submissionList(offset: $offset, limit: $limit, lastKey: $last, questionSlug: $slug) {
            hasNext
            submissions { id statusDisplay lang runtime timestamp }
        }
    }`;
  const list = {};
  for (let i = 0; i < 10; i++) {
    const json = await retry(() => leetcode.graphql({
      query: ql,
      operationName: "Submissions",
      variables: { slug, offset: 0 * 20, limit: 20 }
    }), max_retry);
    const { submissions, hasNext } = json.data.submissionList;
    for (const submission of submissions) {
      const { id, statusDisplay, lang, runtime, timestamp } = submission;
      if (statusDisplay !== "Accepted") {
        continue;
      }
      if (!list[lang] || parseInt(runtime) < parseInt(list[lang].runtime)) {
        list[lang] = { id, timestamp, runtime };
      }
    }
    if (submissions.length === 0 || !hasNext) {
      break;
    }
  }
  return list;
}
async function get_submission(id) {
  var _a;
  const res = await retry(() => (0, import_node_fetch.default)(`${LEETCODE_DETAIL_BASE}${id}/`, {
    headers: {
      Cookie: `LEETCODE_SESSION=${leetcode.credential.session}`,
      Referer: LEETCODE_BASE
    }
  }), max_retry);
  const raw = await res.text();
  const data = (_a = raw.match(/var pageData = ({[^]+?});/)) == null ? void 0 : _a[1];
  const json = new Function("return " + data)();
  const result = {
    runtime: +json.runtime,
    runtime_distribution: json.runtimeDistributionFormatted ? JSON.parse(json.runtimeDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    runtime_percentile: 0,
    memory: +json.memory,
    memory_distribution: json.memoryDistributionFormatted ? JSON.parse(json.memoryDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    memory_percentile: 0,
    code: json.submissionCode
  };
  if (result.runtime_distribution) {
    result.runtime_percentile = result.runtime_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.runtime ? p : 0), 0);
  }
  if (result.memory_distribution) {
    result.memory_percentile = result.memory_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.memory / 1e3 ? p : 0), 0);
  }
  return result;
}
async function get_username() {
  return (await retry(() => leetcode.graphql({
    query: `query globalData { userStatus { username } }`,
    operationName: "globalData",
    variables: {}
  }), max_retry)).data.userStatus.username;
}

// src/index.ts
(0, import_dotenv.config)();
var package_meta = get_package_meta();
import_commander.program.version(import_chalk2.default`{cyanBright ${package_meta.name}} {yellowBright v${package_meta.version}}`);
process.stdout.isTTY = true;
import_commander.program.option("-s, --session <session>", "Your LeetCode Session", process.env.LEETCODE_SESSION || "process.env.LEETCODE_SESSION").option("-o, --output <path>", "Output Dir", import_node_path3.default.resolve("leetcode")).option("-c, --clean", "Clean Output Dir Before Start", true).option("-cd, --cooldown <ms>", "Cooldown Between Actions, in ms", "1000").option("-t, --timezone <timezone>", "Your Timezone", "Asia/Taipei").option("-p, --pure", "Pure Mode, No Additional Informations to Add", false).option("-r, --retry <times>", "Times to Retry When Fail", "3").action(async () => {
  const opts = import_commander.program.opts();
  await dump(opts.session, opts.output, opts.clean, +opts.cooldown, opts.timezone, opts.pure, +opts.retry);
});
import_commander.program.parse();
